local levels = {}
levels.debug_on = 1
levels.collider = {}
levels.top_left_x = 0
levels.top_left_y = 0
levels.tile_width = 32
levels.tile_height = 32
levels.x_edge = 0 
levels.y_edge = 0
levels.sprite_width = 16
levels.sprite_height = 16
levels.data = {}
levels.tileSprites = {}

function levels.update(dt)
  --[[if not levels.sprite_setup then
    levels.sprite_setup = true
    levels.tileSprites = levels.data.load_sheets()
    levels.set_quads(levels.tileSprites)
  end]]
end

function levels.draw()
  love.graphics.setColor(1,1,1,1)
  levels.sprite_draw()
  
  -- Collisions
  if levels.debug_on > 0 then
    love.graphics.setColor(0,1,0,1)
    for _, obj in ipairs(levels.collider) do
      love.graphics.polygon("line",
        obj.body:getWorldPoints(
          obj.shape:getPoints()))
    end
  end
end

function levels.set_sprites(tileSheet_name)
  for index, name in ipairs(tileSheet_name) do 
    local sprite_table = {}
    table.insert(sprite_table, assets[name])
    table.insert(levels.tileSprites, sprite_table)
  end
end

function levels.add_collider(userData,x,y,w,h, friction)
  local collider = {}
  
  collider.body = love.physics.newBody(World, x, y, "static")
  collider.shape = love.physics.newRectangleShape(w/2,h/2, w, h)
  collider.fixture = love.physics.newFixture(collider.body, collider.shape, 1)
  collider.fixture:setUserData(userData)
  collider.fixture:setFriction(friction)

  table.insert(levels.collider, collider)
end

function levels.build_level(level_data)
  levels.data = level_data
  levels.set_sprites(level_data.tileSheets_names)
  levels.set_quads(levels.tileSprites)
  
  -- Fake loop for (Danger)
  local row_index = 1
  while row_index <= #level_data.collider do
    local row = level_data.collider[row_index]
    local col_index = 1
    
    while col_index <= #row do 
      local tile = row[col_index]
      local adjacents = 0
      local userData = nil
      local friction = 0
         
      if tile == 1 then
        userData = "ground"
        friction = 0.2
      end
      if tile == 2 then
        userData = "ground"
        friction = 0.2
      end
      if tile == 3 then
        userData = "ground"
        friction = 0.2
      end
      
      if type(userData) == "string" then
        local adjacents = levels.check_adjacents(row_index, col_index, tile, level_data)
        local new_x = levels.top_left_x + (levels.tile_width * (col_index - 1))
        local new_y = levels.top_left_y + (levels.tile_height * (row_index - 1))
        
        levels.add_collider(userData, new_x, new_y, levels.tile_width + (levels.tile_width * adjacents), levels.tile_height, friction)
        col_index = col_index + adjacents
      end
      col_index = col_index + 1
    end
    row_index = row_index + 1
  end
end

function levels.check_adjacents(new_row_index, new_col_index, tile, level_data)
  local adjacents = 0 
  
  for row_index = new_row_index, #level_data.collider do
    local row = level_data.collider[row_index]
    
    for col_index = new_col_index, #row do 
      local next_tile = row[col_index]
  
      if next_tile == tile then
        adjacents = adjacents + 1
        if col_index == #row then
          return adjacents - 1
        end 
        
      else 
        return adjacents - 1
      end
    end
  end
  
  return adjacents
end

function levels.sprite_draw()
  local spriteMap = levels.data.spriteMap
 
  for row_index, row in ipairs(spriteMap) do
    for col_index, tile in ipairs(row) do 
      local spriteX = (levels.tile_width * (col_index - 1)) + levels.x_edge
      local spriteY = (levels.tile_height * (row_index - 1)) + levels.y_edge
     
      if tile == 1 then -- Normal brick
        levels.set_draw(levels.tileSprites[1][1], levels.tileSprites[1][2][1], spriteX, spriteY)
      end
      if tile == 2 then -- Desert brick
        levels.set_draw(levels.tileSprites[1][1], levels.tileSprites[1][2][2], spriteX, spriteY)
      end
      if tile == 3 then -- Ice brick
        levels.set_draw(levels.tileSprites[1][1], levels.tileSprites[1][2][3], spriteX, spriteY)
      end
    end
  end
end

function levels.set_quads(sprite_table)
  for data_index, sprite_data in ipairs(sprite_table) do 
    local rows = sprite_data[1]:getHeight()/levels.sprite_height
    local cols = sprite_data[1]:getWidth()/levels.sprite_width
    local quads = {}
    
    for row_index=1, rows do 
      for col_index=1, cols do 
        local quad_x = (levels.sprite_width * (col_index - 1)) + levels.x_edge
        local quad_y = (levels.sprite_height * (row_index - 1)) + levels.y_edge
        
        table.insert(quads,
          love.graphics.newQuad(quad_x,quad_y,
            levels.sprite_width, levels.sprite_height,
            sprite_data[1]:getWidth(), sprite_data[1]:getHeight()
            ))
      end
    end
    
    levels.tileSprites[data_index][2] = quads
  end
end 

function levels.set_draw(sheet,quad,x,y)
  love.graphics.draw(sheet,quad,x,y,0,2,2)
end

return levels