local Aloader = {}

Aloader.loading = false
Aloader.loaded = false

--[[
// ASSETS TYPES: audio, font, image.
// ASSETS TYPE PARAMETRES: {nil} or {parametres}

// ASSETS DATA STRUCTURE: ** It is recommended to designate a single file for asset data

assets = {}
assets.names = {}
assets.paths = {}
assets.type = {}
assets.type_parametres = {}

return assets
]]

local old_assets_num = 0 
local loaded_assets = 0 -- Número de assets carregados
local current_assets = {} -- Assets atuais
local draw = nil -- tela de loading
local cache = {} -- caminho de arquivos já carregados
local names = {} -- Nomes dos assets
local paths = {} -- Caminho dos assets
local types = {} -- tipos des assets
local type_parametres = {} -- parâmetros dos tipos de assets 
local timer = 0 -- controle de fluxo
local time_per_asset = 0.5 -- pausa de tempo por asset
local old_BG_color = {} 
local old_color = {}
local old_font = nil
local return_function = {}
local is_return = false

-- Desenho do Aloader
function Aloader.draw()
  if Aloader.loading and draw ~= nil then
    draw()
  end
end 

-- Atualização do Aloder
function Aloader.update(dt)
  if is_return and type(return_function) == "function" then
    is_return = false
    return_function()
  end
  
  if #paths == 0 and Aloader.loading then
    Aloader.loaded = true
  end
  
  if Aloader.loading then
    timer = timer + dt
  end 

  if Aloader.loading and timer > 0.05 then
    -- Carrega os assets
    for i = loaded_assets + 1, #paths do 
      if types[i] == "audio" then
        current_assets[names[i]] = love.audio.newSource(paths[i], type_parametres[i])
      elseif types[i] == "font" then
        current_assets[names[i]] = love.graphics.newFont(paths[i], type_parametres[i])
      elseif types[i] == "image" then
        current_assets[names[i]] = love.graphics.newImage(paths[i], type_parametres[i])
      end
      break
    end
    
    loaded_assets = loaded_assets + 1 
    
    love.timer.sleep(time_per_asset)
    
    if loaded_assets == #paths then
      Aloader.loaded = true
    end
  end
end

-- Configuração do Aloader
function Aloader.conf(draw_function, timer)
  draw = draw_function or Aloader.default_draw
  timer_per_asset = timer or 0.5
end

-- Carrega assets
function Aloader.load_assets(my_assets_data, assets_table, my_return_function)
  Aloader.loading = true
  Aloader.loaded = false
  return_function = my_return_function
  
  local assets_data = deepCopy(my_assets_data)
  local new_cache = deepCopy(my_assets_data.paths)
  local del_names = deepCopy(my_assets_data.names)
  local old_assets = {}
  old_assets_num = 0
  
  for path_index = #assets_data.paths, 1, -1 do
    for cache_index = #cache, 1, -1 do
      if assets_data.paths[path_index] == cache[cache_index] then
        local index = assets_data.names[path_index]
        old_assets[index] = assets_table[index]
        old_assets_num = old_assets_num + 1
        
        table.remove(assets_data.names, path_index)
        table.remove(assets_data.paths, path_index)
        table.remove(assets_data.types, path_index)
        table.remove(assets_data.type_parametres, path_index)
      end
    end
  end
  
  current_assets = old_assets
  loaded_assets = 0
  cache = new_cache
 
  -- Atualiza os arquivos dos assets
  paths = assets_data.paths
  names = assets_data.names
  types = assets_data.types
  type_parametres = assets_data.type_parametres
  
  -- Armazena as configurações gráficas atuais
  old_font = love.graphics.getFont()

  old_color[1], old_color[2], 
  old_color[3], old_color[4] = love.graphics.getColor()
  
  old_BG_color[1], old_BG_color[2],
  old_BG_color[3], old_BG_color[4] = love.graphics.getBackgroundColor()
end

function Aloader.return_assets()
  -- Reset timer and states
  is_return = true
  Aloader.loading = false
  Aloader.loaded = false
  timer = 0
  
  -- Reset default graphics settings
  love.graphics.setColor(old_color)
  love.graphics.setBackgroundColor(old_BG_color)
  love.graphics.setFont(old_font)
  
  -- Return current loaded assets
  return current_assets
end

function Aloader.default_draw()
  local love_font = love.graphics.newFont(12)
  love.graphics.setFont(love_font)
  
  local w, h = love.graphics.getDimensions()
  love.graphics.setColor(1,1,1,1)
  love.graphics.setBackgroundColor(0.1,0.1,0.1,1)
  
  love.graphics.print("LOADING", w/2 - 28, h/2 - 5)
end

function deepCopy(orig)
  local orig_type = type(orig)
  local copy

  if orig_type == 'table' then
    copy = {}
    for orig_key, orig_value in pairs(orig) do
      copy[deepCopy(orig_key)] = deepCopy(orig_value)
    end
    setmetatable(copy, deepCopy(getmetatable(orig)))
  else
    copy = orig
  end

  return copy
end

function Aloader.debug(color,x,y,r,sx,sy,ox,oy,zx,sy)
  
  local debug_info = {
  "Loading: "..tostring(Aloader.loading),
  "Loaded Assets: "..tostring(loaded_assets) + tostring(old_assets_num),
  "Time p/ Asset: "..tostring(timer_per_asset),
 }
  
  local R,G,B,A = love.graphics.getColor()

  love.graphics.setColor(color or {1,1,1,1})
  love.graphics.print(table.concat(debug_info,"\n"),x,y,r,sx,sy,ox,oy,zx,zy)

  love.graphics.setColor(R,G,B,A)
end

return Aloader
