-- Creator: Ralob
-- Last Modification: January 8, 2023
-- Version: alpha
-- Note: Many of the comments were translated by ChatGPT, so I apologize for any errors :]

love.graphics.setDefaultFilter("nearest", "nearest")

-- Resolution setup
local rs = require "libs/resolution_solution"
rs.conf({game_width = 800, game_height = 600, scale_mode = 1})
rs.setMode(rs.game_width, rs.game_height, {fullscreen = true, resizable = true})

-- Assets loader (Created by me :>)
local Aloader = require "libs/Aloader"
assets = {}

local levels = require "level_builder"

-- Classes
require "classes/player"

function love.load()
  love.physics.setMeter(32)
  World = love.physics.newWorld(0,700,true)
  World:setCallbacks(beginContact, endContact, preSolve, postSolve)
  
  -- Configures the asset loader.
  Aloader.conf(loading_draw,0.5)
 
  -- Loads the assets to be used.
  -- Note: The function provided as a parameter to `load_assets()` is called at the end of the loading process, functioning similar to `love.load`.
  local assets_data = require "assets/data/test_room"
  Aloader.load_assets(assets_data, assets,
  function()
    player = Player.new(100,100)
    
    local data = require "levels/test_room"
    levels.build_level(data)
  end
  )
end 

function love.update(dt)
  if Aloader.loaded then
    assets = Aloader.return_assets()
  end
  Aloader.update(dt)
  
  if not Aloader.loading then
    World:update(dt)
    player:update(dt)
    
    levels.update(dt)
  end
end 

local debug_on = 1

function love.draw()
  rs.push()
    local old_y, old_x, old_w, old_h = love.graphics.getScissor()
    love.graphics.setScissor(rs.get_game_zone())
    love.graphics.scale(1.000001,1.000001)
    
   
    Aloader.draw()
    if not Aloader.loading then
      -- BACKGROUND TEMPORÁRIO!
      love.graphics.setColor(0.2,0.2,0.2,1)
      love.graphics.rectangle("fill", 0,0, rs.game_width, rs.game_height)
      
      
      levels.draw()
      player:draw()
    end
    
    love.graphics.setScissor(old_x, old_y, old_w, old_h)
    rs.pop()
    
    -- Debug 
    if debug_on > 0 then
      debug()
      Aloader.debug({1,1,0,1},0,20,0,0.6,0.6)
    end
end 

function love.resize(w, h)
  rs.resize()
end

-- Gamepad Callbacks
function love.joystickpressed(joy,button)
  player:joystickpressed(joy, button)
  
  if button == 7 or button == 11 then
    debug_on = -debug_on
    player.debug_on = -player.debug_on
    levels.debug_on = -levels.debug_on
  end
end 

-- Keyboard Callbacks
function love.keypressed(key)
  player:keypressed(key)
  
  if key == "f1" then
    debug_on = -debug_on
    player.debug_on = -player.debug_on
    levels.debug_on = -levels.debug_on
  end
end

-- Physics Callbacks
function beginContact(a, b, coll)
  player:beginContact(a, b, coll)
end

function endContact(a,b,coll)
  player:endContact(a,b,coll)
end 

function preSolve(a,b,coll)
  player:preSolve(a,b,coll)
end

function postSolve(a,b,coll,c,d)
  player:postSolve(a,b,coll,c,d)
end

-- Loading screen.
function loading_draw()
  love.graphics.setColor(0.2,0.5,0.3,1)
  love.graphics.rectangle("fill",0,0,rs.game_width,rs.game_height)

  love.graphics.setColor(1,1,1,1)
  love.graphics.print("IS LOADING", 5, 5, 0, 2,2)

  love.graphics.setColor(1,1,1,1)
  love.graphics.print("Temporary screen", 5, 50, 0, 1.5, 1.5)
end

function debug()
  love.graphics.setColor(1,1,1,1)
  love.graphics.print(table.concat({
    "World bodies:"..World:getBodyCount(),
    },"\n"),0,0,nil,0.8,0.8)
end
